package org.beigesoft.handler;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;
import java.util.List;
import java.math.BigDecimal;
import java.io.OutputStream;

import org.beigesoft.exception.ExceptionWithCode;
import org.beigesoft.model.IRequestData;
import org.beigesoft.persistable.CsvMethod;
import org.beigesoft.persistable.CsvColumn;
import org.beigesoft.service.ICsvDataRetriever;
import org.beigesoft.service.ISrvDatabase;
import org.beigesoft.service.ISrvOrm;
import org.beigesoft.service.ICsvWriter;

/**
 * <p>Service that retrieves data by given CSV method and retriever,
 *  then writes CSV data to given output stream.</p>
 *
 * @param <RS> platform dependent RDBMS recordset
 * @author Yury Demidenko
 */
public class HndlCsvWriter<RS> implements IHndlFileReportReq {

  /**
   * <p>Database service.</p>
   */
  private ISrvDatabase<RS> srvDatabase;

  /**
   * <p>ORM service.</p>
   **/
  private ISrvOrm<RS> srvOrm;

  /**
   * <p>CSV writer service.</p>
   **/
  private ICsvWriter csvWriter;

  /**
   * <p>Retrievers map.</p>
   **/
  private Map<String, ICsvDataRetriever> retrievers;

  /**
   * <p>Handle file-report request.</p>
   * @param pReqVars Request scoped variables
   * @param pRequestData Request Data
   * @param pSous servlet output stream
   * @throws Exception - an exception
   */
  @Override
  public final void handle(final Map<String, Object> pReqVars,
    final IRequestData pRequestData,
      final OutputStream pSous) throws Exception {
    String csvMethodIdStr = pRequestData.getParameter("csvMethod");
    Long csvMethodId = Long.parseLong(csvMethodIdStr);
    String priceCategoryIdStr = pRequestData.getParameter("priceCategory");
    Long priceCategoryId = Long.parseLong(priceCategoryIdStr);
    pReqVars.put("priceCategoryId", priceCategoryId);
    String unavailablePriceStr = pRequestData.getParameter("unavailablePrice");
    if (unavailablePriceStr != null && !"".equals(unavailablePriceStr)) {
      BigDecimal unavailablePrice = new BigDecimal(unavailablePriceStr);
      pReqVars.put("unavailablePrice", unavailablePrice);
    }
    String optimisticQuantityStr = pRequestData
      .getParameter("optimisticQuantity");
    if (optimisticQuantityStr != null && !"".equals(optimisticQuantityStr)) {
      BigDecimal optimisticQuantity = new BigDecimal(optimisticQuantityStr);
      pReqVars.put("optimisticQuantity", optimisticQuantity);
    }
    CsvMethod csvMethod = null;
    List<List<Object>> data = null;
    ICsvDataRetriever ret = null;
    try {
      this.srvDatabase.setIsAutocommit(false);
      this.srvDatabase.
        setTransactionIsolation(ISrvDatabase.TRANSACTION_READ_UNCOMMITTED);
      this.srvDatabase.beginTransaction();
      csvMethod = getSrvOrm().retrieveEntityById(pReqVars,
        CsvMethod.class, csvMethodId);
      List<CsvColumn> cols = getSrvOrm()
        .retrieveListWithConditions(pReqVars,
          CsvColumn.class, "where ITSOWNER=" + csvMethod.getItsId());
      csvMethod.setColumns(cols);
      ret = this.retrievers.get(csvMethod.getRetrieverName());
      if (ret == null) {
        throw new ExceptionWithCode(ExceptionWithCode.WRONG_PARAMETER,
          "Can't find retriever " + csvMethod.getRetrieverName());
      }
      data = ret.retrieveData(pReqVars);
      this.srvDatabase.commitTransaction();
    } catch (Exception ex) {
      this.srvDatabase.rollBackTransaction();
      throw ex;
    } finally {
      this.srvDatabase.releaseResources();
    }
    if (data != null) {
      this.csvWriter.write(pReqVars, data, csvMethod, pSous);
    }
  }

  //Simple getters and setters:
  /**
   * <p>Getter for retrievers.</p>
   * @return Map<String, ICsvDataRetriever>
   **/
  public final Map<String, ICsvDataRetriever> getRetrievers() {
    return this.retrievers;
  }

  /**
   * <p>Setter for retrievers.</p>
   * @param pRetrievers reference
   **/
  public final void setRetrievers(
    final Map<String, ICsvDataRetriever> pRetrievers) {
    this.retrievers = pRetrievers;
  }

  /**
   * <p>Getter for srvOrm.</p>
   * @return ISrvOrm<RS>
   **/
  public final ISrvOrm<RS> getSrvOrm() {
    return this.srvOrm;
  }

  /**
   * <p>Setter for srvOrm.</p>
   * @param pSrvOrm reference
   **/
  public final void setSrvOrm(final ISrvOrm<RS> pSrvOrm) {
    this.srvOrm = pSrvOrm;
  }

  /**
   * <p>Getter for csvWriter.</p>
   * @return ICsvWriter
   **/
  public final ICsvWriter getCsvWriter() {
    return this.csvWriter;
  }

  /**
   * <p>Setter for csvWriter.</p>
   * @param pCsvWriter reference
   **/
  public final void setCsvWriter(final ICsvWriter pCsvWriter) {
    this.csvWriter = pCsvWriter;
  }

  /**
   * <p>Getter for srvDatabase.</p>
   * @return ISrvDatabase<RS>
   **/
  public final ISrvDatabase<RS> getSrvDatabase() {
    return this.srvDatabase;
  }

  /**
   * <p>Setter for srvDatabase.</p>
   * @param pSrvDatabase reference
   **/
  public final void setSrvDatabase(final ISrvDatabase<RS> pSrvDatabase) {
    this.srvDatabase = pSrvDatabase;
  }
}
