package org.beigesoft.holder;

/*
 * Copyright (c) 2017 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;
import java.util.HashMap;
import java.lang.reflect.Method;

import org.beigesoft.service.IUtlReflection;

/**
 * <p>Holder of RAPI getters for any requested class.</p>
 *
 * @author Yury Demidenko
 */
public class HolderRapiGetters implements IHolderForClassByName<Method> {

  /**
   * <p>Reflection service.</p>
   **/
  private IUtlReflection utlReflection;

  /**
   * <p>Map of classes and their fields names - RAPI getters.</p>
   **/
  private final Map<Class<?>, Map<String, Method>> rapiMethodsMap =
    new HashMap<Class<?>, Map<String, Method>>();

  /**
   * <p>Get thing for given class and thing name.</p>
   * @param pClass a Class
   * @param pFieldName Thing Name
   * @return a thing
   **/
  @Override
  public final Method getFor(final Class<?> pClass, final String pFieldName) {
    Map<String, Method> methMap = this.rapiMethodsMap.get(pClass);
    if (methMap == null) {
      // There is no way to get from Map partially initialized bean
      // in this double-checked locking implementation
      // cause putting to the Map fully initialized bean
      synchronized (this.rapiMethodsMap) {
        methMap = this.rapiMethodsMap.get(pClass);
        if (methMap == null) {
          methMap = new HashMap<String, Method>();
          Method[] methods = getUtlReflection().retrieveMethods(pClass);
          for (Method method : methods) {
            if (method.getName().startsWith("get")) {
              String fldNm = method.getName().substring(3, 4).toLowerCase()
                + method.getName().substring(4);
              methMap.put(fldNm, method);
            }
          }
          //assigning fully initialized object:
          this.rapiMethodsMap.put(pClass, methMap);
        }
      }
    }
    return methMap.get(pFieldName);
  }

  /**
   * <p>Set thing for given class and thing name.</p>
   * @param pThing Thing
   * @param pClass Class
   * @param pFieldName Thing Name
   **/
  @Override
  public final void setFor(final Method pThing,
    final Class<?> pClass, final String pFieldName) {
    synchronized (this.rapiMethodsMap) {
      Map<String, Method> methMap = this.rapiMethodsMap.get(pClass);
      if (methMap == null) {
        methMap = new HashMap<String, Method>();
        this.rapiMethodsMap.put(pClass, methMap);
      }
      methMap.put(pFieldName, pThing);
    }
  }

  //Simple getters and setters:
  /**
   * <p>Getter for utlReflection.</p>
   * @return IUtlReflection
   **/
  public final IUtlReflection getUtlReflection() {
    return this.utlReflection;
  }

  /**
   * <p>Setter for utlReflection.</p>
   * @param pUtlReflection reference
   **/
  public final void setUtlReflection(final IUtlReflection pUtlReflection) {
    this.utlReflection = pUtlReflection;
  }

  /**
   * <p>Getter for rapiMethodsMap.</p>
   * @return final Map<Class<?>, Map<String, Method>>
   **/
  public final Map<Class<?>, Map<String, Method>> getRapiMethodsMap() {
    return this.rapiMethodsMap;
  }
}
