/*
 * Copyright (c) 2016 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

package org.beigesoft.service;

import java.util.Map;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import org.beigesoft.log.ILogger;

/**
 * <p>Standard implementation of I18N service.
 * Use native2ascii -encoding UTF-8 [in-file] [out-file]
 * to convert from UTF-8 to ISO-8859-1.
 * </p>
 *
 * @author Yury Demidenko
 */
public class SrvI18n implements ISrvI18n {

  /**
   * <p>Logger.</p>
   **/
  private ILogger logger;

  /**
   * <p>Messages for default or the first language.</p>
   **/
  private ResourceBundle messages;

  /**
   * <p>Messages for language map.</p>
   **/
  private final Map<String, ResourceBundle> messagesMap =
    new HashMap<String, ResourceBundle>();

  /**
   * <p>Default initializer that load message bundle by default locale.</p>
   **/
  public final void initDefault() {
    try {
      this.messages = null;
      this.messages = ResourceBundle.getBundle("Messages");
    } catch (Exception e) {
      this.logger.error(null, SrvI18n.class,
        " when loading msgs for default locale ", e);
    }
    if (messages != null) {
      this.messagesMap.put(Locale.getDefault().getLanguage(), this.messages);
      this.logger.info(null, SrvI18n.class,
        "Added messages for default locale: " + Locale.getDefault());
    } else {
      //If there is no MessagesBundle[current-locale].properties
      this.logger.error(null, SrvI18n.class,
        "There is no messages for current locale: " + Locale.getDefault());
    }
  }

  /**
   * <p>Add message bundles by given language/country names.</p>
   * @param pLangCountries array of language/country names,
   *   e.g. ["en", "US", "ru", "RU"]
   **/
  public final void add(final String[] pLangCountries) {
    if (pLangCountries != null && pLangCountries.length % 2 == 0) {
      for (int i = 0; i < pLangCountries.length / 2; i++) {
        Locale locale = new Locale(pLangCountries[i * 2],
          pLangCountries[i * 2 + 1]);
        if (!(messages != null && locale.equals(Locale.getDefault()))) {
          ResourceBundle msgs = null;
          try {
            msgs = ResourceBundle.getBundle("Messages", locale);
          } catch (Exception e) {
            this.logger.error(null, SrvI18n.class,
              " when loading msg for locale " + locale, e);
          }
          if (msgs != null) {
            this.messagesMap.put(pLangCountries[i * 2], msgs);
            this.logger.info(null, SrvI18n.class,
              "Added messages for lang/country: " + pLangCountries[i * 2]
                + "/" + pLangCountries[i * 2 + 1]);
            if (this.messages == null) {
              this.messages = msgs;
            }
          } else {
            //If there is no MessagesBundle[current-locale].properties
            this.logger.error(null, SrvI18n.class,
              "There is no messages for lang/country: " + pLangCountries[i * 2]
                + "/" + pLangCountries[i * 2 + 1]);
          }
        } else {
          this.logger.info(null, SrvI18n.class,
            "Messages already added as default for lang/country: "
              + pLangCountries[i * 2] + "/" + pLangCountries[i * 2 + 1]);
        }
      }
    } else {
      String msg = null;
      if (pLangCountries == null) {
        msg =  "is null!";
      } else {
        for (String str : pLangCountries) {
          if (msg == null) {
            msg = str;
          } else {
            msg += "/" + str;
          }
        }
      }
      this.logger.error(null, SrvI18n.class,
        "Parameters language error, pLangCountries: " + msg);
    }
  }

  /**
   * <p>Evaluate message by given key for default, the first language
   * or preferred language stored as thread local variable.</p>
   * @param key of message
   **/
  @Override
  public final String getMsg(final String key) {
    try {
      return messages.getString(key);
    } catch (Exception e) {
      return "[" + key + "]";
    }
  }

  /**
   * <p>Evaluate message by given key for given language.</p>
   * @param pKey key of message
   * @param pLang e.g. "en", "ru", etc.
   **/
  @Override
  public final String getMsg(final String pKey, final String pLang) {
    try {
      ResourceBundle mb = messagesMap.get(pLang);
      if (mb != null) {
        return mb.getString(pKey);
      } else {
        return getMsg(pKey);
      }
    } catch (Exception e) {
      return "[" + pKey + "]-" + pLang;
    }
  }

  //Simple getters and setters:
  /**
   * <p>Geter for logger.</p>
   * @return ILogger
   **/
  public final ILogger getLogger() {
    return this.logger;
  }

  /**
   * <p>Setter for logger.</p>
   * @param pLogger reference
   **/
  public final void setLogger(final ILogger pLogger) {
    this.logger = pLogger;
  }


  /**
   * <p>Getter for messages.</p>
   * @return ResourceBundle
   **/
  public final ResourceBundle getMessages() {
    return this.messages;
  }

  /**
   * <p>Setter for messages.</p>
   * @param pMessages reference
   **/
  public final void setMessages(final ResourceBundle pMessages) {
    this.messages = pMessages;
  }

  /**
   * <p>Getter for messagesMap.</p>
   * @return final Map<String, ResourceBundle>
   **/
  public final Map<String, ResourceBundle> getMessagesMap() {
    return this.messagesMap;
  }
}
