package org.beigesoft.service;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

/**
 * <p>Service that prints a number according given
 * decimal separator, decimal group separator, decimal places after dot,
 * and digits in the group. The number is passed as String with dot decimal
 * separator (or without it) and no group separator,
 * e.g. "12346734.123" or "12667711". This service never does rounding,
 * so if you pass number "1234.2356" and decimal places after dot 3,
 * then result will be "1,234.235". For null or empty string result will be
 * empty string. </p>
 *
 * @author Yury Demidenko
 */
public class SrvNumberToString implements ISrvNumberToString {

  /**
   * <p>Prints number formatted by given decimal separator, decimal group
   * separator and decimal places after dot. Digits in group is 3.</p>
   * @param pNumber e.g. "12146678.12"
   * @param pDigSep decimal separator, e.g. ","
   * @param pDigGrSep decimal group separator, e.g. " "
   * @param pDecPlAfDot decimal places after dot, e.g. 2
   * @return String internationalized number, e.g. "12 146 678,12"
   **/
  @Override
  public final String print(final String pNumber, final String pDigSep,
    final String pDigGrSep, final Integer pDecPlAfDot) {
    return print(pNumber, pDigSep, pDigGrSep, pDecPlAfDot, 3);
  }

  /**
   * <p>Prints number formatted by given decimal separator, decimal group
   * separator, decimal places after dot and digits in group.</p>
   * @param pNumber e.g. "12146678.12"
   * @param pDigSep decimal separator, e.g. ","
   * @param pDigGrSep decimal group separator, e.g. " "
   * @param pDecPlAfDot decimal places after dot, e.g. 2
   * @param pDigitsInGroup Digits in group, e.g. 3
   * @return String internationalized number, e.g. "12 146 678,12"
   **/
  @Override
  public final String print(final String pNumber, final String pDigSep,
    final String pDigGrSep, final Integer pDecPlAfDot,
      final Integer pDigitsInGroup) {
    if (pNumber == null || "".equals(pNumber)) {
      return "";
    }
    int dotIdx = pNumber.indexOf(".");
    String leftWing;
    String rightWing;
    if (dotIdx == -1) {
      leftWing = pNumber;
      rightWing = null;
    } else {
      leftWing = pNumber.substring(0, dotIdx);
      rightWing = pNumber.substring(dotIdx + 1);
    }
    StringBuffer sb = new StringBuffer();
    if (leftWing.startsWith("-")) {
      leftWing = leftWing.substring(1);
      sb.append("-");
    }
    for (int i = 0; i < leftWing.length(); i++) {
      char ch = leftWing.charAt(i);
      sb.append(ch);
      int idxFl = leftWing.length() - i;
      if (pDigitsInGroup == 2) {
        //hard-coded Indian style 12,12,14,334
        if (idxFl == 4) {
          sb.append(pDigGrSep);
        } else {
          idxFl -= 3;
        }
      }
      if (idxFl >= pDigitsInGroup) {
        int gc = idxFl / pDigitsInGroup;
        if (gc > 0) {
          int rem;
          if (gc == 1) {
            rem = idxFl % pDigitsInGroup;
          } else {
            rem = idxFl % (pDigitsInGroup * gc);
          }
          if (rem == 1) {
            sb.append(pDigGrSep);
          }
        }
      }
    }
    if (pDecPlAfDot > 0) {
      sb.append(pDigSep);
      for (int i = 0; i < pDecPlAfDot; i++) {
        if (rightWing != null && rightWing.length() > i) {
          sb.append(rightWing.charAt(i));
        } else {
          sb.append("0");
        }
      }
    }
    return sb.toString();
  }
}
